(function ($) {
  // Public methods.
  var methods = {
    // Init SimpleSpy.
    init: function(s) {
      var displaySelector = 'div.views-simplespy';

      return this.each(function () {
        // 1. Setup.
        // Persistent variables.
        var v = {};

        // Capture a cache of all list items.
        v.list = $(this);
        var originalItems = v.list.children('li').simpleSpy('order', s);
        v.itemsHeight = [];
        originalItems.each(function() {
          v.itemsHeight.push($(this).height());
        });
        v.items = originalItems.clone();
        v.nextIndex = s.limit;
        v.total = v.items.length;

        if (v.total <= s.limit) {
          // There are not enough items to start effects, just exit.
          return;
        }


        // Chop list down to limit li elements.
        originalItems
          .filter(':gt(' + (s.limit - 1) + ')')
          .remove();

        // Set fixed height of wrapper div for improved performance.
        var newHeight = $(displaySelector).outerHeight(true);
        $(displaySelector).css({
          height : newHeight,
          overflow: 'hidden'
        });

        // 2. Effect.
        v.timer = null;
        v.paused = false;
        v.animating = false;

        // Save persistent variables to list and start timer.
        v.s = s;
        $(this)
          .data('simpleSpy', v)
          .simpleSpy('startCountdown');

        if (s.autopause) {
          // Set up mouseenter and mouseleave event handlers for autopause.
          v.list
            .bind('mouseenter.viewsSimpleSpy', function() { $(this).simpleSpy('pause'); })
            .bind('mouseleave.viewsSimpleSpy', function() { $(this).simpleSpy('resume'); });
        }
      });
    },

    play: function(v) {
      var $this = this;
      v.animating = true;
      
      // Insert new item with opacity and height of zero.
      var next = $(v.items[v.nextIndex]).clone().css({
        height : 0,
        opacity : 0,
        display : 'none'
      }).simpleSpy('appendToList', v);
      var old = v.list.children('li').simpleSpy('first', v);

      // Fade the LAST item out
      old.simpleSpy('animateOld', v, function() {
        // We can remove the last item.
        $(this).remove();

        // Increase the height of the new item.
        next.simpleSpy('animateNext', v, function() {
          v.nextIndex++;
          if (v.nextIndex >= v.total) {
            v.nextIndex = 0;
          }

          v.animating = false;

          // Start new iteration.
          $this.simpleSpy('startCountdown');
        });
      });
      return this;
    },

    startCountdown: function(v) {
      var $this = this;
      if (!v.paused && !v.animating) {
        v.timer = setTimeout(function() {
          $this.simpleSpy('play');
        }, v.s.interval);
      }
      return this;
    },

    pause: function(v) {
      v.paused = true;
      clearTimeout(v.timer);
      return this;
    },

    resume: function(v) {
      clearTimeout(v.timer);
      v.paused = false;
      return this.simpleSpy('startCountdown');
    },

    // Functions for "up" direction.
    up: {
      reorder: function(v) {
        return this;
      },

      first: function(v) {
        return this.slice(0, 1);
      },

      appendToList: function(v) {
        return this.appendTo(v.list);
      },

      animateOld: function(v, func) {
        return this
          .animate({opacity : 0}, v.s.fadespeed)
          .animate({height : 0}, v.s.slidespeed, func);
      },

      animateNext: function(v, func) {
        // Skip height animation, because item is transparent and has no influence
        // on other items, so user can't see any effect. Skipping is done by
        // duration set to 0.
        return this
          .animate({height : v.itemsHeight[v.nextIndex]}, 0)
          .animate({opacity : 1}, v.s.fadespeed, func);
      }
    },
    down: {
      reorder: function(v) {
        return $(this.get().reverse());
      },

      first: function(v) {
        return this.slice(-1);
      },

      appendToList: function(v) {
        return this.prependTo(v.list);
      },

      animateOld: function(v, func) {
        // Skip height animation, because item is transparent and has no influence
        // on other items, so user can't see any effect. Skipping is done by
        // duration set to 0.
        return this
          .animate({opacity : 0}, v.s.fadespeed)
          .animate({height : 0}, 0, func);
      },

      animateNext: function(v, func) {
        return this
          .animate({height : v.itemsHeight[v.nextIndex]}, v.s.slidespeed)
          .animate({opacity : 1}, v.s.fadespeed, func);
      }
    }
  };

  $.fn.simpleSpy = function() {
    if (arguments.length == 0 || (arguments.length == 1 && typeof arguments[0] == 'object')) {
      // Init method, we should fill settings with defaults.
      var s = $.extend({
        limit : 4,
        interval : 4000,
        fadespeed : 1000,
        slidespeed : 1000,
        direction : 'down',
        autopause : true
      }, arguments[0]);
      return methods.init.call(this, s);
    }
    else if (arguments.length == 1 && typeof arguments[0] == 'string') {
      // Functions called on SimpleSpy list, they have no arguments, but should
      // receive v from list data.
      var func = arguments[0];
      var v = this.data('simpleSpy');
      if (v && methods[func]) {
        return methods[func].call(this, v);
      }
    }
    else if (arguments.length >= 2 && typeof arguments[0] == 'string' && typeof arguments[1] == 'object') {
      // Direction-dependent functions called on any element with v passed as
      // a first argument.
      var func = arguments[0];
      var v = arguments[1];
      // Get arguments to pass to methods: remove method name from array.
      var args = Array.prototype.slice.call(arguments, 1);
      if (v.s && v.s.direction && methods[v.s.direction] && methods[v.s.direction][func]) {
        return methods[v.s.direction][func].apply(this, args);
      }
    }
    
    // In case of errors do nothing.
    return this;
  };
})(jQuery);


Drupal.behaviors.viewsSimpleSpy = function(context) {
  var contentClass = 'simplespy-content';
  var processedClass = 'simplespy-processed';
  var listSelector = 'ul.' + contentClass + ':not(.' + processedClass + ')'; // used for selecting simplespy content list
  
  $.each(Drupal.settings.viewsSimpleSpy, function(view_id) {
    var displays = this;
    $.each(displays, function(display_id) {
      var contentSelector = 'div.view-id-' + view_id + '.view-display-id-' + display_id + ' ' + listSelector;
      var settings = this;
      $(contentSelector, context)
        .addClass(processedClass)
        .simpleSpy(settings);
      });
  });
}

